@@ -3,13 +3,11 @@
 #
 #  (C) Paul Evans, 2019-2020 -- leonerd@leonerd.org.uk
 
-package Future::AsyncAwait::Awaitable;
+package Future::AsyncAwait::Awaitable 0.44;
 
-use strict;
+use v5.14;
 use warnings;
 
-our $VERSION = '0.42';
-
 =head1 NAME
 
 C<Future::AsyncAwait::Awaitable> - the interface required by C<Future::AsyncAwait>
@@ -167,6 +165,10 @@ method.
 
    sub AWAIT_ON_CANCEL { }
 
+A newer version of this API specification will rename this to
+C<AWAIT_CHAIN_CANCEL>, so it is a good idea to alias the method under both
+names for now.
+
 =cut
 
 =head1 AUTHOR
